﻿using System.IO;

using DevExpress.Images;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Controls;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 루트 노드
        /// </summary>
        private BreadCrumbNode rootNode;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 에디터 버튼을 추가한다.

            EditorButton backButton = new EditorButton()
            {
                Kind    = ButtonPredefines.Glyph,
                Caption = "뒤로",
                Image   = ImageResourceCache.Default.GetImage("office2013/navigation/backward_16x16.png")
            };

            EditorButton addButton = new EditorButton()
            {
                Kind    = ButtonPredefines.Glyph,
                Caption = "이력 추가",
                Image   = ImageResourceCache.Default.GetImage("office2013/actions/add_16x16.png")
            };

            EditorButton historyButton = new EditorButton()
            {
                Kind    = ButtonPredefines.Glyph,
                Caption = "이력 보기",
                Image   = ImageResourceCache.Default.GetImage("office2013/history/historyitem_16x16.png")
            };

            this.breadCrumbEdit.Properties.Buttons.Add(backButton   );
            this.breadCrumbEdit.Properties.Buttons.Add(addButton    );
            this.breadCrumbEdit.Properties.Buttons.Add(historyButton);

            #endregion
            #region 루트 노드를 추가한다.

            this.rootNode = new BreadCrumbNode();

            this.rootNode.Caption          = "내 PC";
            this.rootNode.Value            = "";
            this.rootNode.Persistent       = true;
            this.rootNode.PopulateOnDemand = true;
              
            this.breadCrumbEdit.Properties.Nodes.Add(this.rootNode);

            #endregion
            #region 이벤트를 설정한다.

            this.breadCrumbEdit.Properties.ValidatePath    += RepositoryItemBreadCrumbEdit_ValidatePath;
            this.breadCrumbEdit.Properties.NewNodeAdding   += RepositoryItemBreadCrumbEdit_NewNodeAdding;
            this.breadCrumbEdit.Properties.QueryChildNodes += RepositoryItemBreadCrumbEdit_QueryChildNodes;
            this.breadCrumbEdit.Properties.ButtonClick     += RepositoryItemBreadCrumbEdit_ButtonClick;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 저장소 항목 브레드 크럼 에디터 경로 검증시 처리하기 - RepositoryItemBreadCrumbEdit_ValidatePath(sender, e)
 
        /// <summary>
        /// 저장소 항목 브레드 크럼 에디터 경로 검증시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void RepositoryItemBreadCrumbEdit_ValidatePath(object sender, BreadCrumbValidatePathEventArgs e)
        {
            if(!Directory.Exists(e.Path))
            {
                e.ValidationResult = BreadCrumbValidatePathResult.Cancel;
 
                return;
            }
 
            e.ValidationResult = BreadCrumbValidatePathResult.CreateNodes;
        }
 
        #endregion
        #region 저장소 항목 브레드 크럼 에디터 신규 노드 추가시 처리하기 - RepositoryItemBreadCrumbEdit_NewNodeAdding(sender, e)
 
        /// <summary>
        /// 저장소 항목 브레드 크럼 에디터 신규 노드 추가시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void RepositoryItemBreadCrumbEdit_NewNodeAdding(object sender, BreadCrumbNewNodeAddingEventArgs e)
        {
            e.Node.PopulateOnDemand = true;
        }
 
        #endregion
        #region 저장소 항목 브레드 크럼 에디터 자식 노드 질의시 처리하기 - RepositoryItemBreadCrumbEdit_QueryChildNodes(sender, e)
 
        /// <summary>
        /// 저장소 항목 브레드 크럼 에디터 자식 노드 질의시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void RepositoryItemBreadCrumbEdit_QueryChildNodes(object sender, BreadCrumbQueryChildNodesEventArgs e)
        {
            if(e.Node.Caption == this.rootNode.Caption)
            {
                foreach(DriveInfo driveInfo in DriveInfo.GetDrives())
                {
                    if(driveInfo.DriveType != DriveType.Fixed)
                    {
                        continue;
                    }
 
                    BreadCrumbNode node = new BreadCrumbNode(driveInfo.Name, driveInfo.RootDirectory, true);

                    e.Node.ChildNodes.Add(node);
                }                
                return;
            }

            string directoryPath = e.Node.Path;
 
            if(!Directory.Exists(directoryPath))
            {
                return;
            }
 
            string[] childDirectoryPathArray = Directory.GetDirectories(directoryPath, "*", SearchOption.TopDirectoryOnly);
 
            for(int i = 0; i < childDirectoryPathArray.Length; i++)
            {
                string folderName = new DirectoryInfo(childDirectoryPathArray[i]).Name;

                BreadCrumbNode node = new BreadCrumbNode(folderName, folderName, true);

                e.Node.ChildNodes.Add(node);
            }
        }

        #endregion
        #region 저장소 항목 브레드 크럼 에디터 버튼 클릭시 처리하기 - RepositoryItemBreadCrumbEdit_ButtonClick(sender, e)

        /// <summary>
        /// 저장소 항목 브레드 크럼 에디터 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void RepositoryItemBreadCrumbEdit_ButtonClick(object sender, ButtonPressedEventArgs e)
        {
            BreadCrumbEdit breadCrumbEdit = sender as BreadCrumbEdit;

            switch(e.Button.Caption)
            {
                case "뒤로" :

                    breadCrumbEdit.GoBack();

                    break;

                case "이력 추가" :

                    breadCrumbEdit.Properties.History.Add(new BreadCrumbHistoryItem(breadCrumbEdit.Path));

                    breadCrumbEdit.ClosePopup();

                    break;

                case "이력 보기" :

                    breadCrumbEdit.ShowPopup();

                    break;
            }
        }

        #endregion
    }
}